class ErrorCodes:
    def __init__(self):
        codes = dict()
        codes['-1'] = 'IS_NO_SUCCESS;General error message'
        codes['0'] = 'IS_SUCCESS;Function executed successfully'
        codes[
            '1'] = 'IS_INVALID_CAMERA_HANDLE;Invalid camera handle Most of the uEye SDK functions expect the camera handle as the first parameter.'
        codes[
            '2'] = 'IS_IO_REQUEST_FAILED;An IO request from the uEye driver failed. Possibly the versions of the ueye_api.dll (API) and the driver file (ueye_usb.sys or ueye_eth.sys) do not match.'
        codes[
            '3'] = 'IS_CANT_OPEN_DEVICE;An attempt to initialize or select the camera failed (no camera connected or initialization error).'
        codes['11'] = 'IS_CANT_OPEN_REGISTRY;Error opening a Windows registry key'
        codes['12'] = 'IS_CANT_READ_REGISTRY;Error reading settings from the Windows registry'
        codes['15'] = 'IS_NO_IMAGE_MEM_ALLOCATED;The driver could not allocate memory.'
        codes['16'] = 'IS_CANT_CLEANUP_MEMORY;The driver could not release the allocated memory.'
        codes[
            '17'] = 'IS_CANT_COMMUNICATE_WITH_DRIVER;Communication with the driver failed because no driver has been loaded.'
        codes['18'] = 'IS_FUNCTION_NOT_SUPPORTED_YET;The function is not supported yet.'
        codes['30'] = 'IS_INVALID_IMAGE_SIZE;Invalid image size'
        codes[
            '32'] = 'IS_INVALID_CAPTURE_MODE;The function can not be executed in the current camera operating mode (free run, trigger or standby).'
        codes['49'] = 'IS_INVALID_MEMORY_POINTER;Invalid pointer or invalid memory ID'
        codes['50'] = 'IS_FILE_WRITE_OPEN_ERROR;File cannot be opened for writing or reading.'
        codes['51'] = 'IS_FILE_READ_OPEN_ERROR;The file cannot be opened.'
        codes['52'] = 'IS_FILE_READ_INVALID_BMP_ID;The specified file is not a valid bitmap file.'
        codes['53'] = 'IS_FILE_READ_INVALID_BMP_SIZE;The bitmap size is not correct (bitmap too large).'
        codes[
            '108'] = 'IS_NO_ACTIVE_IMG_MEM;No active image memory available. You must set the memory to active using the is_SetImageMem() function or create a sequence using the is_AddToSequence() function.'
        codes['112'] = 'IS_SEQUENCE_LIST_EMPTY;The sequence list is empty and cannot be deleted.'
        codes[
            '113'] = 'IS_CANT_ADD_TO_SEQUENCE;The image memory is already included in the sequence and cannot be added again.'
        codes[
            '117'] = 'IS_SEQUENCE_BUF_ALREADY_LOCKED;The memory could not be locked. The pointer to the buffer is invalid.'
        codes[
            '118'] = 'IS_INVALID_DEVICE_ID;The device ID is invalid. Valid IDs start from 1 for USB cameras, and from 1001 for GigE cameras.'
        codes['119'] = 'IS_INVALID_BOARD_ID;The board ID is invalid. Valid IDs range from 1 through 255.'
        codes['120'] = 'IS_ALL_DEVICES_BUSY;All cameras are in use.'
        codes[
            '122'] = 'IS_TIMED_OUT;A timeout occurred. An image capturing process could not be terminated within the allowable period.'
        codes['123'] = 'IS_NULL_POINTER;Invalid array'
        codes[
            '125'] = 'IS_INVALID_PARAMETER;One of the submitted parameters is outside the valid range or is not supported for this sensor or is not available in this mode.'
        codes['127'] = 'IS_OUT_OF_MEMORY;No memory could be allocated.'
        codes['129'] = 'IS_ACCESS_VIOLATION;An internal error has occurred.'
        codes[
            '139'] = 'IS_NO_USB20;The camera is connected to a port which does not support the USB 2.0 high-speed standard. Cameras without a memory board cannot be operated on a USB 1.1 port.'
        codes['140'] = 'IS_CAPTURE_RUNNING;A capturing operation is in progress and must be terminated first.'
        codes[
            '145'] = 'IS_IMAGE_NOT_PRESENT;The requested image is not available in the camera memory or is no longer valid.'
        codes[
            '148'] = 'IS_TRIGGER_ACTIVATED;The function cannot be used because the camera is waiting for a trigger signal.'
        codes['151'] = 'IS_CRC_ERROR;A CRC error-correction problem occurred while reading the settings.'
        codes['152'] = 'IS_NOT_YET_RELEASED;This function has not been enabled yet in this version.'
        codes['153'] = 'IS_NOT_CALIBRATED;The camera does not contain any calibration data.'
        codes['154'] = 'IS_WAITING_FOR_KERNEL;The system is waiting for the kernel driver to respond.'
        codes['155'] = 'IS_NOT_SUPPORTED;The camera model used here does not support this function or setting.'
        codes['156'] = 'IS_TRIGGER_NOT_ACTIVATED;The function is not possible as trigger is disabled.'
        codes[
            '157'] = 'IS_OPERATION_ABORTED;The dialog was canceled without a selection so that no file could be saved.'
        codes['158'] = 'IS_BAD_STRUCTURE_SIZE;An internal structure has an incorrect size.'
        codes[
            '159'] = 'IS_INVALID_BUFFER_SIZE;The image memory has an inappropriate size to store the image in the desired format.'
        codes[
            '160'] = 'IS_INVALID_PIXEL_CLOCK;This setting is not available for the currently set pixel clock frequency.'
        codes['161'] = 'IS_INVALID_EXPOSURE_TIME;This setting is not available for the currently set exposure time.'
        codes[
            '162'] = 'IS_AUTO_EXPOSURE_RUNNING;This setting cannot be changed while automatic exposure time control is enabled.'
        codes['163'] = 'IS_CANNOT_CREATE_BB_SURF;The BackBuffer surface cannot be created.'
        codes['164'] = 'IS_CANNOT_CREATE_BB_MIX;The BackBuffer mix surface cannot be created.'
        codes['165'] = 'IS_BB_OVLMEM_NULL;The BackBuffer overlay memory cannot be locked.'
        codes['166'] = 'IS_CANNOT_CREATE_BB_OVL;The BackBuffer overlay memory cannot be created.'
        codes['167'] = 'IS_NOT_SUPP_IN_OVL_SURF_MODE;Not supported in BackBuffer Overlay mode.'
        codes['168'] = 'IS_INVALID_SURFACE;Back buffer surface invalid.'
        codes['169'] = 'IS_SURFACE_LOST;Back buffer surface not found.'
        codes['170'] = 'IS_RELEASE_BB_OVL_DC;Error releasing the overlay device context.'
        codes['171'] = 'IS_BB_TIMER_NOT_CREATED;The back buffer timer could not be created.'
        codes['172'] = 'IS_BB_OVL_NOT_EN;The back buffer overlay was not enabled.'
        codes['173'] = 'IS_ONLY_IN_BB_MODE;Only possible in BackBuffer mode.'
        codes['174'] = 'IS_INVALID_COLOR_FORMAT;Invalid color format'
        codes[
            '175'] = 'IS_INVALID_WB_BINNING_MODE;Mono binning/mono sub-sampling do not support automatic white balance.'
        codes['176'] = 'IS_INVALID_I2C_DEVICE_ADDRESS;Invalid I2C device address'
        codes['177'] = 'IS_COULD_NOT_CONVERT;The current image could not be processed.'
        codes[
            '178'] = 'IS_TRANSFER_ERROR;Transfer error. Frequent transfer errors can mostly be avoided by reducing the pixel rate.'
        codes['179'] = 'IS_PARAMETER_SET_NOT_PRESENT;Parameter set is not present.'
        codes[
            '180'] = 'IS_INVALID_CAMERA_TYPE;The camera type defined in the .ini file does not match the current camera model.'
        codes['181'] = 'IS_INVALID_HOST_IP_HIBYTE;Invalid HIBYTE of host address.'
        codes['182'] = 'IS_CM_NOT_SUPP_IN_CURR_DISPLAYMODE;The color mode is not supported in the current display mode.'
        codes['183'] = 'IS_NO_IR_FILTER;No IR filter available'
        codes[
            '184'] = "IS_STARTER_FW_UPLOAD_NEEDED;The camera's starter firmware is not compatible with the driver and needs to be updated."
        codes['185'] = 'IS_DR_LIBRARY_NOT_FOUND;The DirectRenderer library could not be found.'
        codes['186'] = 'IS_DR_DEVICE_OUT_OF_MEMORY;Not enough graphics memory available.'
        codes['187'] = 'IS_DR_CANNOT_CREATE_SURFACE;The image surface or overlay surface could not be created.'
        codes['188'] = 'IS_DR_CANNOT_CREATE_VERTEX_BUFFER;The vertex buffer could not be created.'
        codes['189'] = 'IS_DR_CANNOT_CREATE_TEXTURE;The texture could not be created.'
        codes['190'] = 'IS_DR_CANNOT_LOCK_OVERLAY_SURFACE;The overlay surface could not be locked.'
        codes['191'] = 'IS_DR_CANNOT_UNLOCK_OVERLAY_SURFACE;The overlay surface could not be unlocked.'
        codes['192'] = 'IS_DR_CANNOT_GET_OVERLAY_DC;Could not get the device context handle for the overlay.'
        codes['193'] = 'IS_DR_CANNOT_RELEASE_OVERLAY_DC;Could not release the device context handle for the overlay.'
        codes['194'] = 'IS_DR_DEVICE_CAPS_INSUFFICIENT;Function is not supported by the graphics hardware.'
        codes['195'] = 'IS_INCOMPATIBLE_SETTING;Because of other incompatible settings the function is not possible.'
        codes['196'] = 'IS_DR_NOT_ALLOWED_WHILE_DC_IS_ACTIVE;A device context handle is still open in the application.'
        codes['197'] = 'IS_DEVICE_ALREADY_PAIRED;The device is already paired.'
        codes['198'] = 'IS_SUBNETMASK_MISMATCH;The subnet mask of the camera and PC network card are different.'
        codes['199'] = 'IS_SUBNET_MISMATCH;The subnet of the camera and PC network card are different.'
        codes['200'] = 'IS_INVALID_IP_CONFIGURATION;The configuration of the IP address is invalid.'
        codes['201'] = 'IS_DEVICE_NOT_COMPATIBLE;The device is not compatible to the drivers.'
        codes[
            '202'] = 'IS_NETWORK_FRAME_SIZE_INCOMPATIBLE;The settings for the image size of the camera are not compatible to the PC network card.'
        codes['203'] = 'IS_NETWORK_CONFIGURATION_INVALID;The configuration of the network card is invalid.'
        codes['204'] = 'IS_ERROR_CPU_IDLE_STATES_CONFIGURATION;The configuration of the CPU idle has failed.'
        codes['205'] = 'IS_DEVICE_BUSY;The camera is busy and cannot transfer the requested image.'
        codes['206'] = 'IS_SENSOR_INITIALIZATION_FAILED;The initialization of the sensor failed.'
        codes['207'] = 'IS_IMAGE_BUFFER_NOT_DWORD_ALIGNED;The image buffer is not DWORD aligned.'
        codes['208'] = 'IS_SEQ_BUFFER_IS_LOCKED;The image memory is locked.'
        codes['209'] = 'IS_FILE_PATH_DOES_NOT_EXIST;The file path does not exist.'
        codes['210'] = 'IS_INVALID_WINDOW_HANDLE;Invalid Window handle'
        self.codes = codes

    def get_code(self, code):
        return self.codes[code] if code in self.codes.keys() else self.codes['-1']



#!/usr/bin/env python

# ------------------------------------------------------------------------------
#                 PyuEye - uEye API High-Level Wrapper
#
# Copyright (c) 2017 by IDS Imaging Development Systems GmbH.
# All rights reserved.
#
# PyuEye is a lean wrapper implementation of Python function objects that
# represent uEye API functions. These bindings could be used as is.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ------------------------------------------------------------------------------

# Title: High-level wrapper for the IDS package pyueye
# Author: João Vital
# Email: joao.vital@frontwave.pt
# Company: Frontwave, S.A. - Pêro Pinheiro, Portugal
# Date: 16 - January - 2019
# Note: Only the most used functions are implemented. More functions may be implemented in future updates.
# References:
#     . pyueye_example_*.py by 	Karl Skretting

from pyueye import ueye as u
from simple_pyueye.ueye_codes import color_mode_info, uEyeException,uEyeError
from PIL import Image
import numpy as np
import win32event as we
import warnings
from time import asctime


class ImageBuffer:
    def __init__(self):
        self.ptr = u.c_mem_p()
        self.id = u.int()


class CameraObj:
    """Quick start:
            camera=Camera(camID)
            camera.open()
            camera.capture_still()
            camera.close()
            """

    def __init__(self, camID, logger=None):
        """
        Initializes the instance with a Camera ID
        :param camID: int
        """
        self.id = u.HIDS(camID)
        self.logger=logger
        self.mem = list()
        self.AOI = {'Width': int(), 'Height': int(), 'X': int(), 'Y': int()}
        self.SensorInfo = u.SENSORINFO()
        self.CameraInfo = u.CAMINFO()
        self.MultiAOIParams = u.AOI_SEQUENCE_PARAMS()
        self.ImageFormat = u.IMAGE_FORMAT_LIST()
        self.EventHandler = list()
        self.nChannels = int()
        self.bits_per_pixel = int()
        self.exposure = float()
        self.gains = [int(), int(), int(), int()]
        self.gamma = int()
        self.frame_rate = float()
        self.currContainer = None
        self.pixel_clock = int()
        self.SerialNumber = int()
        self.capture = bool()

    def check(self, command, log=None):
        """
        Check command execution. If the camera has a logger, if an error raises it will be logged.
        :param command: python command using the ueye library
        :param log: text to log if an error is raised
        :return:
        """
        nRet = command
        if nRet != 0:
            string, description = uEyeError().call(nRet)
            if self.logger is not None:
                if log is not None:
                    self.logger.log(log)

                elif self.logger.text is not None:
                    self.logger.log(self.logger.text)

                else:
                    self.logger.log('{}\n{}, {}, {}\n'.format(asctime(), nRet, string, description))

            raise uEyeException(string, description)
    
    def open(self, verbose=True):
        """
        Opens Camera instance, ready for setting parameters and capturing images
        :param verbose: bool , if True, prints a message confirming the opening of the camera.
        :return:
        """

        self.check(u.is_InitCamera(self.id, None))
        self.get_info()
        self.get_format_list()
        if verbose:
            print('Camera {} connected.'.format(self.id))

    def get_info(self):
        """
        Gathers info over the camera and sensor. Stored in CameraInfo and SensorInfo variables
        :return:
        """
        self.check(u.is_GetCameraInfo(self.id, self.CameraInfo))
        self.check(u.is_GetSensorInfo(self.id, self.SensorInfo))
        self.SerialNumber = int(self.CameraInfo.SerNo)
        self.get_multi_aoi_info()
        self.get_color_mode()
        self.get_aoi()
        """Create container with dimensions of the current camera memory to store an image"""
        self.create_container()

    def get_color_mode(self):
        """
        Gets the code of the current color mode. Sets the bits_per_pixel and nchannels accordingly to allocate memory
        and create containers for the captured images
        :return: color_mode code
        """
        color_mode = u.is_SetColorMode(self.id, u.IS_GET_COLOR_MODE)
        self.bits_per_pixel = color_mode_info(color_mode)[1]
        self.nChannels = color_mode_info(color_mode)[3]
        return color_mode

    def get_aoi(self, verbose=True):
        """
        Gets the information on the current AOI in order to allocate memory and create containers for the captured images
        :param verbose: bool , if true, prints the current AOI settings
        :return:
        """
        currrectAOI = u.IS_RECT()
        self.check(u.is_AOI(self.id, u.IS_AOI_IMAGE_GET_AOI, currrectAOI, u.sizeof(currrectAOI)))
        self.AOI['X'] = int(currrectAOI.s32X)
        self.AOI['Y'] = int(currrectAOI.s32Y)
        self.AOI['Width'] = int(currrectAOI.s32Width)
        self.AOI['Height'] = int(currrectAOI.s32Height)
        if verbose:
            print('current AOI: {}'.format(self.AOI))

    def create_container(self, current=True):
        """
        Creates a container variable for captured images
        :param current: bool , if True, uses an internal variable currContainer to allocate memory for captured images
        :return:
        """
        if current:
            self.get_aoi(verbose=False)
            self.currContainer = np.zeros((self.AOI['Height'], self.AOI['Width'], self.nChannels), dtype=np.uint8)
        else:
            return np.zeros((self.AOI['Height'], self.AOI['Width'], self.nChannels), dtype=np.uint8)

    def set_color_mode(self, color_mode):
        """
        Sets the color mode
        :param color_mode: int, in the form of ueye.IS_CM_...
        :return:
        """
        self.check(u.is_SetColorMode(self.id, color_mode))
        self.bits_per_pixel = color_mode_info(color_mode)[1]
        self.nChannels = color_mode_info(color_mode)[3]
        self.create_container()

    def alloc_mem(self, verbose=True):
        """
        Allocate memory in the camera with a pointer and an id associated
        :return:
        """
        if self.AOI['Width'] == 0:
            self.get_aoi(verbose=False)

        mem = ImageBuffer()
        # self.check(u.is_FreeImageMem(self.id, mem.ptr, mem.id))
        self.check(u.is_AllocImageMem(self.id, self.AOI['Width'], self.AOI['Height'], self.bits_per_pixel, mem.ptr, mem.id))
        self.check(u.is_SetImageMem(self.id, mem.ptr, mem.id))
        self.mem.append(mem)
        if verbose:
            print('Memory Allocated and Set. ID: {}'.format(mem.id))

    def set_aoi(self, width, height, xPos, yPos):
        """
        Sets the AOI (Area of Interest)
        :param width: int
        :param height: int
        :param xPos: int
        :param yPos: int
        :return:
        """
        AOIrect = u.IS_RECT()
        AOIrect.s32Width = u.c_int(width)
        AOIrect.s32Height = u.c_int(height)
        if xPos == 'center':
            AOIrect.s32X = u.c_int((self.SensorInfo.nMaxWidth - width) // 2)
        else:
            AOIrect.s32X = u.c_int(xPos)

        if yPos == 'center':
            AOIrect.s32Y = u.c_int((self.SensorInfo.nMaxHeight - height) // 2)
        else:
            AOIrect.s32Y = u.c_int(yPos)

        self.check(u.is_AOI(self.id, u.IS_AOI_IMAGE_SET_AOI, AOIrect, u.sizeof(AOIrect)))
        """Query the newly set AOI to confirm it's correct setting"""
        self.get_aoi()
        self.create_container()

    def set_exposure(self, value, verbose=True):
        """
        Sets a new value for the exposure time
        :param value: double
        :param verbose: bool, if True, prints the set Exposure
        :return:
        """
        value = u.c_double(value)
        self.check(u.is_Exposure(self.id, u.IS_EXPOSURE_CMD_SET_EXPOSURE, value, u.sizeof(value)))
        self.exposure = value.value
        if verbose:
            print('Exposure set to: {:.3f}'.format(value.value))

    def get_exposure(self):
        """
        Gets the current exposure time
        :return:
        """
        value = u.double()
        self.check(u.is_Exposure(self.id, u.IS_EXPOSURE_CMD_GET_EXPOSURE, value, u.sizeof(value)))
        self.exposure = value.value
        return self.exposure

    def set_fps(self, value, verbose=True):
        """
        Sets a new value for the framerate
        :param value: double
        :param verbose: bool , if True, prints the newly set value
        :return:
        """
        value = u.c_double(value)
        nvalue = u.double()
        self.check(u.is_SetFrameRate(self.id, value, nvalue))
        self.frame_rate = nvalue.value
        if verbose:
            print('Framerate set to {:.3f}'.format(nvalue.value))

    def get_fps(self):
        """
        Gets the current framerate value
        :return:
        """
        value = u.double()
        self.check(u.is_GetFramesPerSecond(self.id, value))
        self.frame_rate = value.value
        return self.frame_rate


    def set_sequence_aoi_mode(self, no_aoi):
        """
        Sets the Sequence AOI mode. self.check if your camera model supports this mode.
        :param no_aoi: int number of AOI, maximum is 4
        :return:
        """
        self.start_sequence(no_aoi)
        nMask = int()
        if no_aoi == 2:
            nMask = u.IS_AOI_SEQUENCE_INDEX_AOI_1 | u.IS_AOI_SEQUENCE_INDEX_AOI_2
        elif no_aoi == 3:
            nMask = u.IS_AOI_SEQUENCE_INDEX_AOI_1 | u.IS_AOI_SEQUENCE_INDEX_AOI_2 | u.IS_AOI_SEQUENCE_INDEX_AOI_3
        elif no_aoi == 4:
            nMask = u.IS_AOI_SEQUENCE_INDEX_AOI_1 | u.IS_AOI_SEQUENCE_INDEX_AOI_2 | u.IS_AOI_SEQUENCE_INDEX_AOI_3 | u.IS_AOI_SEQUENCE_INDEX_AOI_4
        self.check(u.is_AOI(self.id, u.IS_AOI_SEQUENCE_SET_ENABLE, u.c_int(nMask), u.sizeof(u.c_int(nMask))))

    def set_sequence_aoi_params(self, aoi_index, exposure, gain, xPos, yPos, detatch=True, readout=1):
        """
        Sets the parameters for a Sequence AOI
        :param aoi_index: int , in the form of ex: u.IS_AOI_SEQUENCE_INDEX_AOI_2
        :param exposure: double
        :param gain: int
        :param xPos: int
        :param yPos: int
        :param detatch: bool , if True, settings from the main AOI are not automatically copied to the other Sequence AOIs
        :param readout: int , determines how many times the AOI are read. defaults to 1
        :return:
        """
        p = u.AOI_SEQUENCE_PARAMS()
        p.s32AOIIndex = u.c_int(aoi_index)
        p.dblExposure = u.c_double(exposure)
        p.s32Gain = u.c_int(gain)
        p.s32DetatchImageParameters = u.c_int(int(detatch))
        p.s32NumberOfCycleRepetitions = u.c_int(readout)
        p.s32X = u.c_int(xPos)
        p.s32Y = u.c_int(yPos)
        self.check(u.is_AOI(self.id, u.IS_AOI_SEQUENCE_SET_PARAMS, p, u.sizeof(p)))

    def set_pixelclock(self, value, verbose=True):
        """
        Sets a new Pixel Clock value
        :param value: int
        :param verbose: bool , if True, prints the set value
        :return:
        """
        value = u.c_uint(value)
        curr_value = u.c_uint()
        self.check(u.is_PixelClock(self.id, u.IS_PIXELCLOCK_CMD_SET, value, u.sizeof(value)))
        self.check(u.is_PixelClock(self.id, u.IS_PIXELCLOCK_CMD_GET, curr_value, u.sizeof(curr_value)))
        self.pixel_clock = curr_value.value
        if verbose:
            print('PixelClock set to: {}'.format(curr_value))

    def set_hardware_gains(self, values, verbose=True):
        """
        Sets values for the hardware gains
        :param values: list of int, in the form of [MASTER, RED_GAIN, GREEN_GAIN, BLUE_GAIN].
                If you do not wish to change one of the entries, set to ueye.IS_IGNORE_PARAMETER or -1
        :param verbose: bool, if True, print the set gains
        :return:
        """
        master = u.c_int(values[0])
        r_gain = u.c_int(values[1])
        g_gain = u.c_int(values[2])
        b_gain = u.c_int(values[3])
        self.check(u.is_SetHardwareGain(self.id, master, r_gain, g_gain, b_gain))
        self.gains = [master.value, r_gain.value, g_gain.value, b_gain.value]
        if verbose:
            print('Set gains: {}'.format(values))

    def set_gamma(self, value, verbose=True):
        """
        Sets a new gamma value
        :param value: int
        :param verbose: bool , if True, prints the set value
        :return:
        """
        value = u.c_int(int(value))
        self.check(u.is_Gamma(self.id, u.IS_GAMMA_CMD_SET, value, u.sizeof(value)))
        self.gamma = value.value
        if verbose:
            print('Gamma set to: {}'.format(value))

    def install_event(self, event):
        """
        Installs an event and appends it to the EventHandler variable
        :param event: int, code of the event, in the form of ueye.IS_EVENT...
        :return:
        """
        hEv_py = we.CreateEvent(None, 0, 0, None)
        hEv = u.c_void_p(int(hEv_py))
        self.check(u.is_InitEvent(self.id, hEv, event))
        self.check(u.is_EnableEvent(self.id, event))
        self.EventHandler.append(hEv_py)

    def set_trigger_mode(self, trigger_mode, verbose=True):
        """
        Sets a trigger mode
        :param trigger_mode: int, trigger mode code, in the form of ueye.IS_SET_TRIGGER...
        :param verbose: bool , if True, prints a message confirming the success of the operation
        :return:
        """
        self.check(u.is_SetExternalTrigger(self.id, trigger_mode))
        if verbose:
            print('Set Trigger Mode')

    def set_prescaler(self, value, verbose=True):
        """
        Sets a value for the prescaler. The camera will be triggered every 'value' signals.
        i.e. The camera skips 'value' signals
        :param value: int, prescaler value to set
        :param verbose: bool, if True, prints the set value
        :return:
        """
        value = u.c_uint(value)
        self.check(u.is_Trigger(self.id, u.IS_TRIGGER_CMD_SET_FRAME_PRESCALER, value, u.sizeof(value)))
        if verbose:
            print('Prescaler set to: {}'.format(value.value))

    def close(self, verbose=True):
        """
        Closes the camera
        :param verbose: bool , if True, prints a message confirming the success of the operation.
        :return:
        """
        for mem in self.mem:
            self.check(u.is_UnlockSeqBuf(self.id, mem.id, mem.ptr))
            self.check(u.is_FreeImageMem(self.id, mem.ptr, mem.id))
        self.check(u.is_ExitCamera(self.id))
        if verbose:
            print('Camera {} disconnected'.format(self.id))

    def get_format_list(self):
        """
        Retrieves the possible predefined image formats
        :return:
        """
        count = u.c_uint()
        self.check(u.is_ImageFormat(self.id, u.IMGFRMT_CMD_GET_NUM_ENTRIES, count, u.sizeof(count)))
        pformatList = u.IMAGE_FORMAT_LIST(FormatInfo=(u.IMAGE_FORMAT_INFO * count))
        pformatList.nNumListElements = count
        pformatList.nSizeOfListEntry = u.sizeof(u.IMAGE_FORMAT_INFO)
        self.check(u.is_ImageFormat(self.id, u.IMGFRMT_CMD_GET_LIST, pformatList, u.sizeof(pformatList)))
        self.ImageFormat = pformatList

    def set_image_format(self, format_id):
        """
        Sets a predefined image format. self.check the available formats in the uEye Cockpit
        :param format_id: int, format coe number
        :return:
        """
        formatInfo = self.ImageFormat.FormatInfo[format_id]
        self.check(u.is_ImageFormat(self.id, u.IMGFRMT_CMD_SET_FORMAT, formatInfo.nFormatID, u.sizeof(formatInfo.nFormatID)))
        self.get_aoi()

    def get_multi_aoi_info(self):
        """
        Initializes the structures needed for multi AOI.
        :return:
        """
        m_nMaxNumberMultiAOIs = u.uint()
        self.check(u.is_AOI(self.id, u.IS_AOI_MULTI_GET_AOI | u.IS_AOI_MULTI_MODE_GET_MAX_NUMBER, m_nMaxNumberMultiAOIs,
                       u.sizeof(m_nMaxNumberMultiAOIs)))
        rMinSizeAOI = u.IS_SIZE_2D()
        self.check(u.is_AOI(self.id, u.IS_AOI_MULTI_GET_AOI | u.IS_AOI_MULTI_MODE_GET_MINIMUM_SIZE, rMinSizeAOI,
                       u.sizeof(rMinSizeAOI)))
        if m_nMaxNumberMultiAOIs > 1:
            m_psMultiAOIs = u.IS_MULTI_AOI_CONTAINER(pMultiAOIList=(u.IS_MULTI_AOI_DESCRIPTOR * m_nMaxNumberMultiAOIs))
            m_psMultiAOIs.nNumberOfAOIs = m_nMaxNumberMultiAOIs
            self.MultiAOIParams = m_psMultiAOIs

    def set_multi_aoi_param(self, aoi_index, xPos, yPos, width, height):
        """
        Set the parameters of a specified AOI. Keep in mind that the camera concatenates all AOIs in the MultiAOIList,
        hence the positions and sizes must be so that the set forms a rectangle.
        See documentation for further clarification.
        :param aoi_index: int , number of the AOI to set the parameters for
        :param xPos: int
        :param yPos: int
        :param width: int
        :param height: int
        :return:
        """
        self.MultiAOIParams.pMultiAOIList[aoi_index].nPosX = u.c_uint(xPos)
        self.MultiAOIParams.pMultiAOIList[aoi_index].nPosY = u.c_uint(yPos)
        self.MultiAOIParams.pMultiAOIList[aoi_index].nWidth = u.c_uint(width)
        self.MultiAOIParams.pMultiAOIList[aoi_index].nHeight = u.c_uint(height)
        self.MultiAOIParams.pMultiAOIList[aoi_index].nStatus = u.c_uint(u.IS_AOI_MULTI_STATUS_SETBYUSER)

    def start_multi_aoi(self):
        """
        Verifies the MultiAOI settings and sets them if everything is OK.
        :return:
        """
        self.check(u.is_AOI(self.id, u.IS_AOI_MULTI_SET_AOI | u.IS_AOI_MULTI_MODE_ONLY_VERIFY_AOIS, self.MultiAOIParams,
                       u.sizeof(u.IS_MULTI_AOI_CONTAINER()) + u.sizeof(
                           u.IS_MULTI_AOI_DESCRIPTOR()) * self.MultiAOIParams.nNumberOfAOIs))
        self.check(u.is_AOI(self.id, u.IS_AOI_MULTI_SET_AOI, self.MultiAOIParams, u.sizeof(self.MultiAOIParams)))
        self.create_container()

    def start_sequence(self, no_buffs):
        """
        Starts a sequence of 'no_buffs' buffers. This is important to ensure all frames are captured even if there is a
        delay of more than the capturing time between two frames.
        :param no_buffs: int, number of buffers to allocate
        :return:
        """
        for i in range(no_buffs):
            mem = ImageBuffer()
            self.check(u.is_AllocImageMem(self.id, self.AOI['Width'], self.AOI['Height'], self.bits_per_pixel, mem.ptr,
                                     mem.id))
            self.mem.append(mem)

        for mem in self.mem:
            self.check(u.is_AddToSequence(self.id, mem.ptr, mem.id))

        self.check(u.is_InitImageQueue(self.id, 0))

    def set_trigger_debounce(self, mode='auto', delay=None, verbose=True):
        """
        :param mode: str, defaults to auto, can be ['auto', 'rising', 'falling', 'both']
        :param delay: int, sets the time the signal has to be on to be considered
        :param verbose: bool, whether or not to display a message confirming the operation
        :return:
        """
        if mode == 'auto':
            self.check(u.is_TriggerDebounce(self.id, u.TRIGGER_DEBOUNCE_CMD_SET_MODE,
                                       u.c_uint(u.TRIGGER_DEBOUNCE_MODE_AUTOMATIC),
                                       u.sizeof(u.c_uint(u.TRIGGER_DEBOUNCE_MODE_AUTOMATIC))))

        elif mode == 'rising':
            self.check(u.is_TriggerDebounce(self.id, u.TRIGGER_DEBOUNCE_CMD_SET_MODE, u.TRIGGER_DEBOUNCE_MODE_RISING_EDGE,
                                       u.sizeof(u.TRIGGER_DEBOUNCE_MODE_RISING_EDGE)))

        elif mode == 'falling':
            self.check(u.is_TriggerDebounce(self.id, u.TRIGGER_DEBOUNCE_CMD_SET_MODE, u.TRIGGER_DEBOUNCE_MODE_FALLING_EDGE,
                                       u.sizeof(u.TRIGGER_DEBOUNCE_MODE_FALLING_EDGE)))

        elif mode == 'both':
            self.check(u.is_TriggerDebounce(self.id, u.TRIGGER_DEBOUNCE_CMD_SET_MODE, u.TRIGGER_DEBOUNCE_MODE_BOTH_EDGES,
                                       u.sizeof(u.TRIGGER_DEBOUNCE_MODE_BOTH_EDGES)))

        else:
            self.check(u.is_TriggerDebounce(self.id, u.TRIGGER_DEBOUNCE_CMD_SET_MODE, u.TRIGGER_DEBOUNCE_MODE_NONE,
                                       u.sizeof(u.TRIGGER_DEBOUNCE_MODE_NONE)))
        if verbose:
            print('Trigger debounce mode set to: {}'.format(mode))

        if delay is not None:
            delay = u.c_uint(int(delay))
            self.check(u.is_TriggerDebounce(self.id, u.TRIGGER_DEBOUNCE_CMD_SET_DELAY_TIME, delay, u.sizeof(delay)))
            if verbose:
                print('Trigger debounce delay set successfully to: {}'.format(delay))

    def stop_live_video(self, verbose=False):
        self.check(u.is_StopLiveVideo(self.id, u.IS_DONT_WAIT))
        if verbose:
            print('Live video from camera {} stopped...'.format(self.id))

    def execute(self,command):
        """
        Execute a command and wrap it with this object's logger
        :param command:
        :return:
        """
        self.check(command)

    def capture_still(self, save=False, filename='sample.png'):
        """
        Captures a still picture
        :param save: bool , if True, image is saved to a file, if not, a numpy array will be returned containing the image
        :param filename: string , with the path to the file where the captured image will be stored
        :return:
        """
        if self.mem.__len__() == 0:
            warnings.warn('Warning: Memory not allocated! Allocating memory to default AOI...')
            self.alloc_mem()

        self.check(u.is_FreezeVideo(self.id, u.IS_WAIT))
        mem = ImageBuffer()
        self.check(u.is_GetActiveImageMem(self.id, mem.ptr, mem.id))
        self.check(u.is_CopyImageMem(self.id, mem.ptr, mem.id, self.currContainer.ctypes.data))
        if save:
            im = Image.fromarray(self.currContainer)
            im.save(filename)
        return self.currContainer

    def continuous_capture(self):
        """
        Run this preferably inside a Thread.
        :return:
        """
        if self.mem.__len__() == 0:
            warnings.warn('Warning: Memory not allocated! Allocating memory to default AOI...')
            self.alloc_mem()

        if self.EventHandler.__len__() == 0:
            warnings.warn('Warning: Frame Event not installed! Installing...')
            self.install_event(u.IS_FRAME)

        self.capture = True
        self.check(u.is_CaptureVideo(self.id, u.IS_WAIT))
        while self.capture:
            mem = ImageBuffer()
            we.WaitForSingleObject(self.EventHandler[0], we.INFINITE)
            self.check(u.is_GetActiveImageMem(self.id, mem.ptr, mem.id))
            self.check(u.is_CopyImageMem(self.id, mem.ptr, mem.id, self.currContainer.ctypes.data))
            self.check(u.is_UnlockSeqBuf(self.id, mem.id, mem.ptr))

            """Place further processing code here
                Image is placed in self currContainer"""

        self.stop_live_video()


if __name__ == '__main__':
    from simple_pyueye.ueye_codes import Logger
    logger = Logger('C:/stonescanchapa/temp/sys.info')
    cam = CameraObj(0, logger=logger)
    cam.execute(u.is_InitCamera(cam.id,None))
    # cam.open()
    # cam.capture_still(save=True, filename='img.png')
    cam.close()



from simple_pyueye.camera import *
from threading import Thread
import cv2 as cv


def still_capture(camID=1):
    camera = CameraObj(camID)
    camera.open()
    camera.capture_still()
    camera.close()


class ImageProcess:
    def __init__(self):
        self.type = 'None'
        self.emboss_filter = np.array([[-2, 1, 0], [-1, 1, 1], [0, 1, 2]])

    def filter(self, img, normalize=False):
        if self.type == 'None':
            pass
        elif self.type == 'laplacian':
            img = cv.Laplacian(img, cv.CV_64F, 7)
        elif self.type == 'emboss':
            img = cv.filter2D(img, cv.CV_8U, self.emboss_filter)
        elif self.type == 'invert':
            img = np.abs(img.astype(np.int) - 255)
        if normalize:
            img = self.normalize(img)
        return img.clip(0, 255).astype(np.uint8)

    def normalize(self, img):
        return ((img - img.min()) / (img.max() - img.min())) * 255


def continuous_capture_filters(camera):
    if camera.mem.__len__() == 0:
        warnings.warn('Warning: Memory not allocated! Allocating memory to default AOI...')
        camera.alloc_mem()

    if camera.EventHandler.__len__() == 0:
        warnings.warn('Warning: Frame Event not installed! Installing...')
        camera.install_event(u.IS_FRAME)

        camera.capture = True
    check(u.is_CaptureVideo(camera.id, u.IS_WAIT))

    ip = ImageProcess()
    while camera.capture:
        mem = ImageBuffer()
        we.WaitForSingleObject(camera.EventHandler[0], we.INFINITE)
        check(u.is_GetActiveImageMem(camera.id, mem.ptr, mem.id))
        check(u.is_CopyImageMem(camera.id, mem.ptr, mem.id, camera.currContainer.ctypes.data))
        check(u.is_UnlockSeqBuf(camera.id, mem.id, mem.ptr))

        """Place further processing code here
            Image is placed in self currContainer"""

        play = ip.filter(camera.currContainer)
        size = cv.getTextSize('Press q to exit', cv.FONT_HERSHEY_SIMPLEX, 1, 1)
        cv.rectangle(play, (10, 50 - size[0][1]), (10 + size[0][0], 50 + size[1]), (0, 0, 0),
                     thickness=cv.FILLED)
        cv.putText(play, 'Press q to exit', (10, 50), cv.FONT_HERSHEY_SIMPLEX, 1, (255, 255, 255),
                   thickness=1)

        key = cv.waitKey(1)
        if key == ord('q'):
            camera.capture = False
        elif key == ord('l'):
            ip.type = 'laplacian'
        elif key == ord('e'):
            ip.type = 'emboss'
        elif key == ord('n'):
            ip.type = 'None'
        elif key == ord('i'):
            ip.type = 'invert'

        # play = (((play - play.min()) / (play.max() - play.min())) * 255).astype(np.uint8)
        cv.imshow('Live', play)
    camera.stop_live_video()


def continuous_capture(camID=1):
    camera = CameraObj(camID)
    camera.open()
    # camera.set_color_mode(u.IS_CM_MONO8)
    t = Thread(target=continuous_capture_filters, args=(camera,))
    t.start()
    t.join()
    camera.close()



"""
Usage: python setup.py sdist bdist_wheel
"""
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fr:
    requirements = fr.read().splitlines()

setuptools.setup(
    name="simple_pyueye",
    version="0.0.6",
    author="Joao Vital",
    author_email="joao.vital@frontwave.com",
	license='3-Clause BSD License',
    description="High-level wrapper for the python bindings package pyueye",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://frontwave.pt/",
    packages=setuptools.find_packages(),
    install_requires=requirements,
)



import numpy as np
from pyueye import ueye as u
import simple_pyueye
import os
from simple_pyueye.error_codes import ErrorCodes
from time import asctime

error_path = simple_pyueye.__file__
err = os.path.sep.join(error_path.split(os.path.sep)[:-1])


class Logger:
    def __init__(self, fp, text=None):
        self.text = text
        self.fp = fp

    def log(self, code):
        with open(self.fp, 'a') as f:
            f.write(str(code) + '\n')


def check(command, logger=None, log=None):
    nRet = command
    if nRet != 0:
        string, description = uEyeError().call(nRet)
        if logger is not None:
            if logger.text is not None:
                logger.log(logger.text)
            elif log is None:
                logger.log('{}\n{}, {}, {}\n'.format(asctime(), nRet, string, description))
            else:
                logger.log(log)
        raise uEyeException(string, description)


class uEyeException(Exception):
    def __init__(self, string, description):
        self.string = string
        self.description = description

    def __str__(self):
        return "Error: {} \nDescription: {}".format(self.string, self.description)


class uEyeError:
    # def __init__(self):
    #     filename = err + '\\error_codes.txt'
    #     error_str = list()
    #     error_description = list()
    #     error_no = list()
    #     f = open(filename, 'r')
    #     for line in f:
    #         val = line.split(';')
    #         error_no.append(int(val[0]))
    #         error_str.append(val[1])
    #         error_description.append(val[2])
    #     error_no = np.array(error_no)
    #     self.error_no = error_no
    #     self.error_str = error_str
    #     self.error_description = error_description

    def call(self, err_no):
        # arg = int(np.argwhere(self.error_no == err_no))
        # string = self.error_str[arg]
        string = str(err_no) + ' ' + ErrorCodes().get_code(str(err_no)).split(';')[0]
        description = ErrorCodes().get_code(str(err_no), ).split(';')[1]
        # description = self.error_description[arg]
        return string, description


def color_mode_info(color_mode):
    """[bits/channel, bits/pixel, total bits, nChannels]"""
    return {u.IS_CM_SENSOR_RAW16: [16, 16, 16, 1],
            u.IS_CM_SENSOR_RAW12: [12, 16, 16, 1],
            u.IS_CM_SENSOR_RAW10: [10, 16, 16, 1],
            u.IS_CM_SENSOR_RAW8: [8, 8, 8, 1],
            u.IS_CM_MONO16: [16, 16, 16, 1],
            u.IS_CM_MONO12: [12, 16, 16, 1],
            u.IS_CM_MONO10: [10, 16, 16, 1],
            u.IS_CM_MONO8: [8, 8, 8, 1],
            u.IS_CM_RGBA12_UNPACKED: [12, 36, 64, 4],
            u.IS_CM_RGB12_UNPACKED: [12, 36, 48, 3],
            u.IS_CM_RGB10_UNPACKED: [10, 30, 48, 3],
            u.IS_CM_RGB10_PACKED: [10, 30, 32, 3],
            u.IS_CM_RGBA8_PACKED: [8, 24, 32, 4],
            u.IS_CM_RGBY8_PACKED: [8, 32, 32, 4],
            u.IS_CM_RGB8_PACKED: [8, 24, 24, 3],
            u.IS_CM_BGRA12_UNPACKED: [12, 36, 64, 4],
            u.IS_CM_BGR12_UNPACKED: [12, 36, 48, 3],
            u.IS_CM_BGR10_UNPACKED: [10, 30, 48, 3],
            u.IS_CM_BGR10_PACKED: [10, 30, 32, 3],
            u.IS_CM_BGRA8_PACKED: [8, 24, 32, 4],
            u.IS_CM_BGRY8_PACKED: [8, 32, 32, 4],
            u.IS_CM_BGR8_PACKED: [8, 24, 24, 3],
            u.IS_CM_BGR565_PACKED: [5, 16, 16, 3],
            u.IS_CM_BGR5_PACKED: [5, 15, 16, 3],
            u.IS_CM_UYVY_PACKED: [8, 16, 16, 4],
            u.IS_CM_UYVY_MONO_PACKED: [8, 16, 16, 4],
            u.IS_CM_UYVY_BAYER_PACKED: [8, 16, 16, 4],
            u.IS_CM_CBYCRY_PACKED: [8, 16, 16, 4]}[color_mode]


if __name__ == '__main__':
    logger = Logger('C:/stonescanchapa/temp/sys.info')
    check(11, logger=logger)
